@@ -1049,7 +1049,12 @@ func processAlign(inp io.Reader, args []string) {
 
 		switch args[0] {
 		case "-g":
-			pdg = eutils.GetNumericArg(args, "-g spacing between columns", 0, 1, 30)
+			if len(args) > 1 && args[1] == "t" {
+				// special case to use tab character between columns
+				pdg = -1
+			} else {
+				pdg = eutils.GetNumericArg(args, "-g spacing between columns", 0, 1, 30)
+			}
 			args = args[2:]
 		case "-h":
 			mrg = eutils.GetNumericArg(args, "-i indent before columns", 0, 1, 30)
