@@ -1,4 +1,4 @@
-#!/bin/sh
+#!/bin/bash
 
 # ===========================================================================
 #
@@ -32,7 +32,14 @@
 #
 # ==========================================================================
 
-version="19.2"
+# environment variable turns on shell tracing
+
+if [ -n "${EDIRECT_TRACE}" ] && [ "${EDIRECT_TRACE}" = true ]
+then
+  set -x
+fi
+
+version="23.8"
 
 # initialize common flags
 
@@ -44,8 +51,12 @@ api_key=""
 immediate=false
 express=false
 
-# undocumented -quick argument blocks isPreview
-# flags that work around PubMed SOLR preview server limits
+reldate=""
+mindate=""
+maxdate=""
+datetype=""
+
+# undocumented -quick argument bypasses logic that works around PubMed SOLR server limits
 quick=false
 quickx=false
 
@@ -63,13 +74,19 @@ debugx=false
 log=false
 logx=false
 
-timer=false
+# timer now on by default
+timer=true
 timerx=false
 
 label=""
 labels=""
 labelx=""
 
+reldatex=""
+mindatex=""
+maxdatex=""
+datetypex=""
+
 verbose=false
 tranquil=false
 
@@ -103,6 +120,20 @@ stp=0
 rest=""
 qury=""
 
+# environment variable disables retry attempts
+
+showErrors=true
+if [ -n "${EDIRECT_NO_ERRORS}" ] && [ "${EDIRECT_NO_ERRORS}" = true ]
+then
+  showErrors=false
+fi
+
+retryAllowed=true
+if [ -n "${EDIRECT_NO_RETRY}" ] && [ "${EDIRECT_NO_RETRY}" = true ]
+then
+  retryAllowed=false
+fi
+
 # set up colors for error report
 
 ColorSetup() {
@@ -132,10 +163,41 @@ ColorSetup() {
   fi
   LOUD="${INIT}${RED}${BOLD}"
   INVT="${LOUD}${FLIP}"
+  # clear color on terminal if "export EDIRECT_TRACE=true" has been used
+  echo "${INIT}" > /dev/null
 }
 
 ColorSetup
 
+# highlighted error and warning functions
+
+DisplayError() {
+
+  if [ $# -gt 0 ]
+  then
+    msg="$1"
+    echo "${INVT} ERROR: ${LOUD} ${msg}${INIT}" >&2
+  fi
+}
+
+DisplayWarning() {
+
+  if [ $# -gt 0 ]
+  then
+    msg="$1"
+    echo "${INVT} WARNING: ${LOUD} ${msg}${INIT}" >&2
+  fi
+}
+
+DisplayNote() {
+
+  if [ $# -gt 0 ]
+  then
+    msg="$1"
+    echo "${INVT} NOTE: ${LOUD} ${msg}${INIT}" >&2
+  fi
+}
+
 # parse ENTREZ_DIRECT, eSearchResult, eLinkResult, or ePostResult
 
 ParseMessage() {
@@ -203,8 +265,10 @@ ParseStdin() {
     mssg=$( cat )
     ParseMessage "$mssg" ENTREZ_DIRECT \
                   dbase Db web_env WebEnv qry_key QueryKey qury Query \
-                  num Count stp Step toolx Tool emailx Email \
-                  labelx Labels quickx Quick debugx Debug logx Log timerx Elapsed
+                  mindatex MinDate maxdatex MaxDate reldatex RelDate \
+                  datetypex DateType num Count \
+                  stp Step toolx Tool emailx Email labelx Labels \
+                  quickx Quick debugx Debug logx Log timerx Elapsed
     if [ "$?" = 2 ]
     then
       # if no ENTREZ_DIRECT message present, support passing raw UIDs via stdin
@@ -251,144 +315,125 @@ ParseStdin() {
 
 # process common control flags
 
+CheckForArgumentValue() {
+
+  tag="$1"
+  rem="$2"
+
+  if [ "$rem" -lt 2 ]
+  then
+    DisplayError "Missing ${tag} argument"
+    exit 1
+  fi
+}
+
 ParseCommonArgs() {
 
   argsConsumed=0
   while [ $# -gt 0 ]
   do
-    case "$1" in
+    tag="$1"
+    rem="$#"
+    case "$tag" in
       -dev )
         dev=true
-        argsConsumed=$(($argsConsumed + 1))
+        argsConsumed=$((argsConsumed + 1))
         shift
         ;;
       -raw )
         raw=true
-        argsConsumed=$(($argsConsumed + 1))
+        argsConsumed=$((argsConsumed + 1))
         shift
         ;;
       -internal | -int )
         internal=true
-        argsConsumed=$(($argsConsumed + 1))
+        argsConsumed=$((argsConsumed + 1))
         shift
         ;;
       -external | -ext )
         external=true
-        argsConsumed=$(($argsConsumed + 1))
+        argsConsumed=$((argsConsumed + 1))
         shift
         ;;
       -immediate )
         immediate=true
-        argsConsumed=$(($argsConsumed + 1))
+        argsConsumed=$((argsConsumed + 1))
         shift
         ;;
       -express )
         express=true
-        argsConsumed=$(($argsConsumed + 1))
+        argsConsumed=$((argsConsumed + 1))
         shift
         ;;
       -base )
-        argsConsumed=$(($argsConsumed + 1))
+        argsConsumed=$((argsConsumed + 1))
+        CheckForArgumentValue "$tag" "$rem"
+        shift
+        basx="$1"
+        argsConsumed=$((argsConsumed + 1))
         shift
-        if [ $# -gt 0 ]
-        then
-          basx="$1"
-          argsConsumed=$(($argsConsumed + 1))
-          shift
-        else
-          echo "${INVT} ERROR: ${LOUD} Missing -base argument${INIT}" >&2
-          exit 1
-        fi
         ;;
       -input )
-        argsConsumed=$(($argsConsumed + 1))
+        argsConsumed=$((argsConsumed + 1))
+        CheckForArgumentValue "$tag" "$rem"
+        shift
+        input="$1"
+        argsConsumed=$((argsConsumed + 1))
         shift
-        if [ $# -gt 0 ]
-        then
-          input="$1"
-          argsConsumed=$(($argsConsumed + 1))
-          shift
-        else
-          echo "${INVT} ERROR: ${LOUD} Missing -input argument${INIT}" >&2
-          exit 1
-        fi
         ;;
       -web )
-        argsConsumed=$(($argsConsumed + 1))
+        argsConsumed=$((argsConsumed + 1))
+        CheckForArgumentValue "$tag" "$rem"
+        shift
+        web_env="$1"
+        argsConsumed=$((argsConsumed + 1))
         shift
-        if [ $# -gt 0 ]
-        then
-          web_env="$1"
-          shift
-        else
-          echo "${INVT} ERROR: ${LOUD} Missing -web argument${INIT}" >&2
-          exit 1
-        fi
         ;;
       -step )
-        argsConsumed=$(($argsConsumed + 1))
+        argsConsumed=$((argsConsumed + 1))
+        CheckForArgumentValue "$tag" "$rem"
+        shift
+        stp="$1"
+        argsConsumed=$((argsConsumed + 1))
         shift
-        if [ $# -gt 0 ]
-        then
-          stp="$1"
-          shift
-        else
-          echo "${INVT} ERROR: ${LOUD} Missing -step argument${INIT}" >&2
-          exit 1
-        fi
         ;;
       -label )
-        argsConsumed=$(($argsConsumed + 1))
+        argsConsumed=$((argsConsumed + 1))
+        CheckForArgumentValue "$tag" "$rem"
+        shift
+        stp="$1"
+        argsConsumed=$((argsConsumed + 1))
         shift
-        if [ $# -gt 0 ]
-        then
-          label="$1"
-          argsConsumed=$(($argsConsumed + 1))
-          shift
-        else
-          echo "${INVT} ERROR: ${LOUD} Missing -label argument${INIT}" >&2
-          exit 1
-        fi
         ;;
       -email )
-        argsConsumed=$(($argsConsumed + 1))
+        argsConsumed=$((argsConsumed + 1))
+        CheckForArgumentValue "$tag" "$rem"
+        shift
+        emailr="$1"
+        argsConsumed=$((argsConsumed + 1))
         shift
-        if [ $# -gt 0 ]
-        then
-          emailr="$1"
-          argsConsumed=$(($argsConsumed + 1))
-          shift
-        else
-          echo "${INVT} ERROR: ${LOUD} Missing -email argument${INIT}" >&2
-          exit 1
-        fi
         ;;
       -tool )
-        argsConsumed=$(($argsConsumed + 1))
+        argsConsumed=$((argsConsumed + 1))
+        CheckForArgumentValue "$tag" "$rem"
+        shift
+        toolr="$1"
+        argsConsumed=$((argsConsumed + 1))
         shift
-        if [ $# -gt 0 ]
-        then
-          toolr="$1"
-          argsConsumed=$(($argsConsumed + 1))
-          shift
-        else
-          echo "${INVT} ERROR: ${LOUD} Missing -tool argument${INIT}" >&2
-          exit 1
-        fi
         ;;
       -quick )
-        argsConsumed=$(($argsConsumed + 1))
+        argsConsumed=$((argsConsumed + 1))
         shift
         if [ $# -gt 0 ]
         then
           if [ "$1" = "true" ]
           then
-            argsConsumed=$(($argsConsumed + 1))
+            argsConsumed=$((argsConsumed + 1))
             shift
             quick=true
           elif [ "$1" = "false" ]
           then
-            argsConsumed=$(($argsConsumed + 1))
+            argsConsumed=$((argsConsumed + 1))
             shift
             quick=false
           else
@@ -399,18 +444,18 @@ ParseCommonArgs() {
         fi
         ;;
       -debug )
-        argsConsumed=$(($argsConsumed + 1))
+        argsConsumed=$((argsConsumed + 1))
         shift
         if [ $# -gt 0 ]
         then
           if [ "$1" = "true" ]
           then
-            argsConsumed=$(($argsConsumed + 1))
+            argsConsumed=$((argsConsumed + 1))
             shift
             debug=true
           elif [ "$1" = "false" ]
           then
-            argsConsumed=$(($argsConsumed + 1))
+            argsConsumed=$((argsConsumed + 1))
             shift
             debug=false
           else
@@ -421,18 +466,18 @@ ParseCommonArgs() {
         fi
         ;;
        -verbose )
-        argsConsumed=$(($argsConsumed + 1))
+        argsConsumed=$((argsConsumed + 1))
         shift
         if [ $# -gt 0 ]
         then
           if [ "$1" = "true" ]
           then
-            argsConsumed=$(($argsConsumed + 1))
+            argsConsumed=$((argsConsumed + 1))
             shift
             verbose=true
           elif [ "$1" = "false" ]
           then
-            argsConsumed=$(($argsConsumed + 1))
+            argsConsumed=$((argsConsumed + 1))
             shift
             verbose=false
           else
@@ -444,22 +489,32 @@ ParseCommonArgs() {
         ;;
       -tranquil )
         tranquil=true
-        argsConsumed=$(($argsConsumed + 1))
+        argsConsumed=$((argsConsumed + 1))
+        shift
+        ;;
+      -clear )
+        quick=false
+        quickx=false
+        debug=false
+        debugx=false
+        verbose=false
+        tranquil=false
+        argsConsumed=$((argsConsumed + 1))
         shift
         ;;
      -log )
-        argsConsumed=$(($argsConsumed + 1))
+        argsConsumed=$((argsConsumed + 1))
         shift
         if [ $# -gt 0 ]
         then
           if [ "$1" = "true" ]
           then
-            argsConsumed=$(($argsConsumed + 1))
+            argsConsumed=$((argsConsumed + 1))
             shift
             log=true
           elif [ "$1" = "false" ]
           then
-            argsConsumed=$(($argsConsumed + 1))
+            argsConsumed=$((argsConsumed + 1))
             shift
             log=false
           else
@@ -470,18 +525,18 @@ ParseCommonArgs() {
         fi
         ;;
       -timer )
-        argsConsumed=$(($argsConsumed + 1))
+        argsConsumed=$((argsConsumed + 1))
         shift
         if [ $# -gt 0 ]
         then
-          if [ "$1" = "true" ]
+          if [ "$1" = "true" ] || [ "$1" = "on" ]
           then
-            argsConsumed=$(($argsConsumed + 1))
+            argsConsumed=$((argsConsumed + 1))
             shift
             timer=true
-          elif [ "$1" = "false" ]
+          elif [ "$1" = "false" ] || [ "$1" = "off" ]
           then
-            argsConsumed=$(($argsConsumed + 1))
+            argsConsumed=$((argsConsumed + 1))
             shift
             timer=false
           else
@@ -496,7 +551,7 @@ ParseCommonArgs() {
         exit 0
         ;;
       -newmode | -oldmode )
-        argsConsumed=$(($argsConsumed + 1))
+        argsConsumed=$((argsConsumed + 1))
         shift
         ;;
       * )
@@ -592,6 +647,18 @@ FinishSetup() {
   # -email argument overrides calculated email, and Email XML field, if read later
 
   AdjustEmailAndTool
+
+  # temporarily remove email name starting with "root@", which crashes server
+  if [ -n "$email" ]
+  then
+    case "$email" in
+      "root@"* )
+        email=""
+        ;;
+      * )
+        ;;
+    esac
+  fi
 }
 
 # prints query command with double quotes around multi-word arguments
@@ -611,7 +678,7 @@ PrintQuery() {
     num=$( printf "%s" "$elm" | wc -w | tr -cd 0-9 )
     [ "$fmt" = "%s" ] || elm=$( printf "$fmt" "$elm" )
     case "$elm:$num:$fmt" in
-      *[^\\][\'\"]:*:%q )
+      *[!\\][\'\"]:*:%q )
         ;;
       *:1:* )
         elm=$( printf "%s" "$raw" | LC_ALL=C sed -e 's/\([]!-*<>?[\\]\)/\\\1/g' )
@@ -659,7 +726,7 @@ ErrorTail() {
   else
     printf "${RED}${msg}${INIT}\n" >&2
   fi
-  if [ "$goOn" = true ]
+  if [ "$goOn" = true ] && [ "$retryAllowed" = true ]
   then
     printf "${BLUE}${whc} ATTEMPT" >&2
   else
@@ -681,7 +748,7 @@ RequestWithRetry() {
   whch="SECOND"
   while [ "$goOn" = true ]
   do
-    tries=$(( $tries - 1 ))
+    tries=$(( tries - 1 ))
     if [ "$tries" -lt 1 ]
     then
       goOn=false
@@ -690,88 +757,124 @@ RequestWithRetry() {
     case "$res" in
       "" )
         # empty result
-        ErrorHead "$warn" "$when"
-        PrintQuery "$@"
-        ErrorTail "EMPTY RESULT" "$whch"
+        if [ "$showErrors" = true ]
+        then
+          ErrorHead "$warn" "$when"
+          PrintQuery "$@"
+          ErrorTail "EMPTY RESULT" "$whch"
+        fi
         sleep 1
         when=$( date )
-        # retry query
-        res=$( "$@" )
+        if [ "$retryAllowed" = true ]
+        then
+          # retry query
+          res=$( "$@" )
+        fi
         ;;
       *\<eFetchResult\>* | *\<eSummaryResult\>*  | *\<eSearchResult\>*  | *\<eLinkResult\>* | *\<ePostResult\>* | *\<eInfoResult\>* )
         case "$res" in
           *\<ERROR\>* )
             ref=$( echo "$res" | transmute -format indent -doctype "" )
-            ErrorHead "$warn" "$when"
-            PrintQuery "$@"
-            if [ "$goOn" = true ]
+            if [ "$showErrors" = true ]
             then
-              # asterisk prints entire selected XML subregion
-              ref=$( echo "$res" | xtract -pattern ERROR -element "*" )
-            fi
-            ErrorTail "$ref" "$whch"
-            sleep 1
-            when=$( date )
-            # retry query
-            res=$( "$@" )
-            ;;
-          *\<error\>* )
-            ref=$( echo "$res" | transmute -format indent -doctype "" )
-            ErrorHead "$warn" "$when"
-            PrintQuery "$@"
-            if [ "$goOn" = true ]
-            then
-              # asterisk prints entire selected XML subregion
-              ref=$( echo "$res" | xtract -pattern error -element "*" )
-            fi
-            ErrorTail "$ref" "$whch"
-            sleep 1
-            when=$( date )
-            # retry query
-            res=$( "$@" )
-            ;;
-          *\<ErrorList\>* )
-            ref=$( echo "$res" | transmute -format indent -doctype "" )
-            # question mark prints names of heterogeneous child objects
-            errs=$( echo "$res" | xtract -pattern "ErrorList/*" -element "?" )
-            if [ -n "$errs" ] && [ "$errs" = "PhraseNotFound" ]
-            then
-              goOn=false
-            else
               ErrorHead "$warn" "$when"
               PrintQuery "$@"
               if [ "$goOn" = true ]
               then
-                # reconstruct indented ErrorList XML
-                ref=$( echo "$res" | xtract -head "<ErrorList>" -tail "<ErrorList>" \
-                       -pattern "ErrorList/*" -pfx "  " -element "*" )
+                # asterisk prints entire selected XML subregion
+                ref=$( echo "$res" | xtract -pattern ERROR -element "*" )
               fi
               ErrorTail "$ref" "$whch"
-              sleep 1
-              when=$( date )
+            fi
+            sleep 1
+            when=$( date )
+            if [ "$retryAllowed" = true ]
+            then
               # retry query
               res=$( "$@" )
             fi
             ;;
-          *\"error\":* )
+          *\<error\>* )
             ref=$( echo "$res" | transmute -format indent -doctype "" )
-            ErrorHead "$warn" "$when"
-            PrintQuery "$@"
-            ErrorTail "$ref" "$whch"
+            if [ "$showErrors" = true ]
+            then
+              ErrorHead "$warn" "$when"
+              PrintQuery "$@"
+              if [ "$goOn" = true ]
+              then
+                # asterisk prints entire selected XML subregion
+                ref=$( echo "$res" | xtract -pattern error -element "*" )
+              fi
+              ErrorTail "$ref" "$whch"
+            fi
             sleep 1
             when=$( date )
-            # retry query
-            res=$( "$@" )
+            if [ "$retryAllowed" = true ]
+            then
+              # retry query
+              res=$( "$@" )
+            fi
+            ;;
+          *\<ErrorList\>* )
+            ref=$( echo "$res" | transmute -format indent -doctype "" )
+            # question mark prints names of heterogeneous child objects
+            errs=$( echo "$res" | xtract -pattern "ErrorList/*" -element "?" | sort -f | uniq -i )
+            if [ -n "$errs" ] && [ "$errs" = "PhraseNotFound" ]
+            then
+              goOn=false
+            else
+              if [ "$showErrors" = true ]
+              then
+                ErrorHead "$warn" "$when"
+                PrintQuery "$@"
+                if [ "$goOn" = true ]
+                then
+                  # reconstruct indented ErrorList XML
+                  ref=$( echo "$res" | xtract -head "<ErrorList>" -tail "<ErrorList>" \
+                         -pattern "ErrorList/*" -pfx "  " -element "*" )
+                fi
+                ErrorTail "$ref" "$whch"
+              fi
+              sleep 1
+              when=$( date )
+              if [ "$retryAllowed" = true ]
+              then
+                # retry query
+                res=$( "$@" )
+              fi
+            fi
+            ;;
+          *\"error\":* )
+            ref=$( echo "$res" | transmute -format indent -doctype "" )
+            if [ "$showErrors" = true ]
+            then
+              ErrorHead "$warn" "$when"
+              PrintQuery "$@"
+              ErrorTail "$ref" "$whch"
+            fi
+            sleep 1
+            when=$( date )
+            if [ "$retryAllowed" = true ]
+            then
+              # retry query
+              res=$( "$@" )
+            fi
             ;;
           *"<DocumentSummarySet status=\"OK\"><!--"* )
             # 'DocSum Backend failed' message embedded in comment
-            ErrorHead "$warn" "$when"
-            PrintQuery "$@"
-            ErrorTail "$res" "$whch"
+            if [ "$showErrors" = true ]
+            then
+              ErrorHead "$warn" "$when"
+              PrintQuery "$@"
+              ErrorTail "$res" "$whch"
+            fi
             sleep 1
             when=$( date )
-            # retry query
-            res=$( "$@" )
+            if [ "$retryAllowed" = true ]
+            then
+              # retry query
+              res=$( "$@" )
+            fi
             ;;
           *\<WarningList\>* )
             case "$res" in
@@ -782,8 +885,49 @@ RequestWithRetry() {
                   goOn=false
                 else
                   ref=$( echo "$res" | transmute -format indent -doctype "" )
-                  # question mark prints names of heterogeneous child objects
-                  errs=$( echo "$res" | xtract -pattern "WarningList/*" -element "?" )
+                  # errs=$( echo "$res" | xtract -pattern "WarningList/*" -element "?" )
+                  if [ "$showErrors" = true ]
+                  then
+                    ErrorHead "$warn" "$when"
+                    PrintQuery "$@"
+                    if [ "$goOn" = true ]
+                    then
+                      # reconstruct indented ErrorList XML
+                      ref=$( echo "$res" | xtract -head "<WarningList>" -tail "<WarningList>" \
+                             -pattern "WarningList/*" -pfx "  " -element "*" )
+                    fi
+                    ErrorTail "$ref" "$whch"
+                  fi
+                  sleep 1
+                  when=$( date )
+                  if [ "$retryAllowed" = true ]
+                  then
+                    # retry query
+                    res=$( "$@" )
+                  fi
+                fi
+                ;;
+              *"<OutputMessage>Wildcard search for"*"used only the first 600 variations"* )
+                if [ "$tranquil" = true ]
+                then
+                  # -tranquil flag conditionally ignores wildcard search root too short message
+                  goOn=false
+                else
+                  goOn=false
+                  if [ "$showErrors" = true ]
+                  then
+                    ErrorHead "$warn" "$when"
+                    errs=$( echo "$res" | xtract -pattern "WarningList/*" -element OutputMessage )
+                    ErrorTail "$errs" "$whch"
+                  fi
+                  sleep 1
+                fi
+                ;;
+              * )
+                ref=$( echo "$res" | transmute -format indent -doctype "" )
+                # errs=$( echo "$res" | xtract -pattern "WarningList/*" -element "?" )
+                if [ "$showErrors" = true ]
+                then
                   ErrorHead "$warn" "$when"
                   PrintQuery "$@"
                   if [ "$goOn" = true ]
@@ -793,30 +937,15 @@ RequestWithRetry() {
                            -pattern "WarningList/*" -pfx "  " -element "*" )
                   fi
                   ErrorTail "$ref" "$whch"
-                  sleep 1
-                  when=$( date )
+                fi
+                sleep 1
+                when=$( date )
+                if [ "$retryAllowed" = true ]
+                then
                   # retry query
                   res=$( "$@" )
                 fi
                 ;;
-              * )
-                ref=$( echo "$res" | transmute -format indent -doctype "" )
-                # question mark prints names of heterogeneous child objects
-                errs=$( echo "$res" | xtract -pattern "WarningList/*" -element "?" )
-                ErrorHead "$warn" "$when"
-                PrintQuery "$@"
-                if [ "$goOn" = true ]
-                then
-                  # reconstruct indented ErrorList XML
-                  ref=$( echo "$res" | xtract -head "<WarningList>" -tail "<WarningList>" \
-                         -pattern "WarningList/*" -pfx "  " -element "*" )
-                fi
-                ErrorTail "$ref" "$whch"
-                sleep 1
-                when=$( date )
-                # retry query
-                res=$( "$@" )
-                ;;
             esac
             ;;
           * )
@@ -827,13 +956,19 @@ RequestWithRetry() {
         ;;
       *"<DocumentSummarySet status=\"OK\"><!--"* )
         # docsum with comment not surrounded by wrapper
-        ErrorHead "$warn" "$when"
-        PrintQuery "$@"
-        ErrorTail "$res" "$whch"
+        if [ "$showErrors" = true ]
+        then
+          ErrorHead "$warn" "$when"
+          PrintQuery "$@"
+          ErrorTail "$res" "$whch"
+        fi
         sleep 1
         when=$( date )
-        # retry query
-        res=$( "$@" )
+        if [ "$retryAllowed" = true ]
+        then
+          # retry query
+          res=$( "$@" )
+        fi
         ;;
       * )
         # success for non-structured or non-EUtils-XML result
@@ -841,6 +976,10 @@ RequestWithRetry() {
         ;;
     esac
     whch="LAST"
+    if [ "$retryAllowed" = false ]
+    then
+      goOn=false
+    fi
   done
 
   # use printf percent-s instead of echo to prevent unwanted evaluation of backslash
@@ -859,6 +998,43 @@ RunWithLogging() {
   RequestWithRetry "$@"
 }
 
+# ensure date constraint argument consistency
+
+FixDateConstraints() {
+
+  if [ -z "$datetype" ]
+  then
+    datetype="PDAT"
+  fi
+
+  if [ -z "$reldate" ] || [ "$reldate" -lt 1 ]
+  then
+    reldate=""
+  fi
+
+  # set default value for missing date range endpoint
+  if [ -n "$mindate" ] && [ -z "$maxdate" ]
+  then
+    currentDate=$(date +%Y)
+    maxdate="$currentDate"
+  fi
+  if [ -z "$mindate" ] && [ -n "$maxdate" ]
+  then
+    mindate="1900"
+  fi
+
+  if [ -z "$mindate" ] || [ -z "$maxdate" ]
+  then
+    mindate=""
+    maxdate=""
+  fi
+
+  if [ -z "$reldate" ] && [ -z "$mindate" ] && [ -z "$maxdate" ]
+  then
+    datetype=""
+  fi
+}
+
 # helpers for constructing argument arrays
 
 AddIfNotEmpty() {
@@ -900,119 +1076,6 @@ RunWithCommonArgs() {
   RunWithLogging "$@"
 }
 
-# break PubMed SOLR server requests into chunks
-
-# time esearch -db pubmed -query "PNAS [JOUR] AND 2020:2022 [PDAT]" | efetch -format uid -verbose > /dev/null
-
-SearchSOLR() {
-
-  local fr
-  local to
-  local md
-  local nx
-
-  qy="$1"
-  fr="$2"
-  to="$3"
-
-  if [ "$verbose" = false ]
-  then
-    printf "." >&2
-  fi
-
-  # -tranquil flag specifically suppresses "No items found" message and automatic retries
-  count=$( esearch -db pubmed -count "( ${qy} AND ${fr}:${to} [UID])" -tranquil < /dev/null )
-  if [ -n "$count" ]
-  then
-    if [ "$count" -eq 0 ]
-    then
-      if [ "$verbose" = true ]
-      then
-        printf "  %08d : %08d [UID] . %d\n" "$fr" "$to" "$count" >&2
-      fi
-    elif [ "$count" -le 9999 ]
-    then
-      if [ "$verbose" = true ]
-      then
-        printf "  %08d : %08d [UID] * %d\n" "$fr" "$to" "$count" >&2
-      fi
-      esearch -db pubmed -subset "( ${qy} AND ${fr}:${to} [UID])" -tranquil < /dev/null |
-      xtract -pattern eSearchResult -block IdList -sep "\n" -element Id |
-      sort -n | uniq
-    else
-      if [ "$verbose" = true ]
-      then
-        printf "  %08d : %08d [UID] - %d\n" "$fr" "$to" "$count" >&2
-      fi
-      # partition half way between the points
-      md=$(( fr / 2 + to / 2 ))
-      nx=$(( md + 1 ))
-      SearchSOLR "$qy" "$fr" "$md"
-      SearchSOLR "$qy" "$nx" "$to"
-    fi
-  else
-    if [ "$verbose" = true ]
-    then
-      printf "  %08d : %08d [UID] ~ 0\n" "$fr" "$to" >&2
-    fi
-  fi
-}
-
-GenerateFromSOLR() {
-
-  total="$num"
-
-  if [ "$total" -lt 1 ]
-  then
-    exit 0
-  fi
-
-  # comment out this conditional section to test with small samples
-  if [ "$total" -lt 10000 ]
-  then
-    esearch -db pubmed -uids "$qury" -tranquil < /dev/null
-    exit 0
-  fi
-
-  minPMID=1
-
-  # remove pubmed books inserted at front of list, but still retrieve 1000 PMIDs and take maximum
-  maxPMID=$(
-    esearch -db pubmed -subset "all [SB] NOT pubmed books [SB]" -retmax 1000 -tranquil < /dev/null |
-    xtract -pattern eSearchResult -max Id
-  )
-
-  # reality check for successful creation of maxPMID variable
-  if [ ! -n "$maxPMID" ]
-  then
-    echo "Unable to get top PMID" >&2
-    exit 0
-  fi
-
-  # occasional older records, newly found and submitted by publishers,
-  # will be lower in the sort order (EDAT is set to its PDAT value), so
-  # increment by 1000 to accommodate this rare situation
-  maxPMID=$(( maxPMID + 1000 ))
-
-  uids=$( SearchSOLR "$qury" "$minPMID" "$maxPMID" | sort -n | uniq )
-
-  if [ "$verbose" = false ]
-  then
-    printf "\n" >&2
-  fi
-
-  if [ -n "$uids" ]
-  then
-    cumulative=$( echo "$uids" | wc -l | tr -d ' ' )
-    if [ "$verbose" = true ]
-    then
-      echo "cumulative: ${cumulative},  total: ${total}" >&2
-    fi
-  fi
-
-  echo "$uids"
-}
-
 # break Entrez history server requests into chunks
 
 GenerateHistoryChunks() {
@@ -1023,7 +1086,7 @@ GenerateHistoryChunks() {
 
   if [ "$minn" -gt 0 ]
   then
-    minn=$(( $minn - 1 ))
+    minn=$(( minn - 1 ))
   fi
   if [ "$maxx" -eq 0 ]
   then
@@ -1034,114 +1097,94 @@ GenerateHistoryChunks() {
 
   while [ "$fr" -lt "$maxx" ]
   do
-    to=$(( $fr + $chnk ))
+    to=$(( fr + chnk ))
     if [ "$to" -gt "$maxx" ]
     then
-      chnk=$(( $maxx - $fr ))
+      chnk=$(( maxx - fr ))
     fi
     echo "$fr" "$chnk"
-    fr=$(( $fr + $chnk ))
+    fr=$(( fr + chnk ))
   done
 }
 
-# return UID list from specified source or Entrez history server
+# passes all possible data source arguments to ecollect
 
-GenerateUidList() {
+CallEcollect() {
 
-  dbsx="$1"
+  # execute query
+  res=$( "$@" )
 
-  fromSOLR=false
-  if [ "$needHistory" = true ]
+  # use printf percent-s instead of echo to prevent unwanted evaluation of backslash
+  printf "%s\n" "$res" | sed -e '${/^$/d;}'
+}
+
+GetFromEcollect() {
+
+#  ecollect -base "$base" -db "$dbase" -web "$web_env" -key "$qry_key" \
+#    -query "$qury" -id "$ids" -rest "$rest" -input "$input" -num "$num"
+
+  set ecollect
+
+  AddIfNotEmpty -base "$base" \
+  AddIfNotEmpty -db "$dbase" \
+  AddIfNotEmpty -web "$web_env" \
+  AddIfNotEmpty -key "$qry_key" \
+  AddIfNotEmpty -query "$qury" \
+  AddIfNotEmpty -reldate "$reldate" \
+  AddIfNotEmpty -mindate "$mindate" \
+  AddIfNotEmpty -maxdate "$maxdate" \
+  AddIfNotEmpty -datetype "$datetype" \
+  AddIfNotEmpty -id "$ids" \
+  AddIfNotEmpty -rest "$rest" \
+  AddIfNotEmpty -input "$input" \
+  AddIfNotEmpty -num "$num" \
+  CallEcollect "$@"
+}
+
+GetUIDs() {
+
+  if [ -n "$web_env" ] && [ -n "$qry_key" ]
   then
-    chunk=25000
-    mxx=0
-    if [ "$dbsx" = "pubmed" ]
-    then
-      chunk=10000
-      if [ "$quick" = false ]
-      then
-        mxx=9999
-      fi
-    fi
-    GenerateHistoryChunks "$chunk" 0 "$mxx" |
-    while read fr chnk
-    do
-      RunWithCommonArgs nquire -url "$base" efetch.fcgi \
-        -query_key "$qry_key" -WebEnv "$web_env" -retstart "$fr" -retmax "$chnk" \
-        -db "$dbsx" -rettype uilist -retmode text
-    done
+    GetFromEcollect
+  elif [ -n "$qury" ] && [ "$dbase" = "pubmed" ]
+  then
+    GetFromEcollect
+  elif [ -n "$ids" ]
+  then
+    # LookupSpecialAccessions instantiates converted accessions into $ids variable,
+    # so -id argument, if populated, must be used before $rest and $input
+    echo "$ids" |
+    # faster version of accn-at-a-time without case transformation
+    tr -cs a-zA-Z0-9_. '\n'
+
+  elif [ -n "$rest" ]
+  then
+    # raw UIDs or instantiated UIDs extracted from ENTREZ_DIRECT message
+    echo "$rest" |
+    tr -cs a-zA-Z0-9_. '\n'
+
+  elif [ -n "$input" ]
+  then
+    # input file of raw UIDs
+    cat "$input" |
+    tr -cs a-zA-Z0-9_. '\n'
+
   else
-    # otherwise obtain raw UIDs from available sources
-    if [ -n "$qury" ] && [ "$dbsx" = "pubmed" ] && [ "$isPreview" = true ]
-    then
-      fromSOLR=true
-      slr=$( GenerateFromSOLR )
-      if [ -n "$slr" ]
-      then
-        nm=$( echo "$slr" | wc -l  | tr -d ' ' )
-        if [ -n "$nm" ] && [ -n "$num" ] && [ "$nm" -ne "$num" ]
-        then
-          echo "${INVT} WARNING: ${LOUD} Actual PMID count ${nm} does not match expected total ${num}${INIT}" >&2
-        fi
-      fi
-      echo "$slr"
-    elif [ -n "$ids" ]
-    then
-      echo "$ids"
-    elif [ -n "$rest" ]
-    then
-      echo "$rest"
-    elif [ -n "$input" ]
-    then
-      cat "$input"
-    fi |
-    # accn-at-a-time without case transformation
-    sed 's/[^a-zA-Z0-9_.]/ /g; s/^ *//' |
-    fmt -w 1 |
-    grep '.'
+    DisplayError "Missing argument describing data source"
+    exit 1
   fi |
+
+  # sort and unique final UID results
   sort -n | uniq
 }
 
 # special case accession to UID lookup functions
 
-ExtractUIDs() {
-
-  if [ "$needHistory" = false ]
-  then
-    GenerateUidList "$dbase" |
-    while read uid
-    do
-      notInteger=$( echo "$uid" | sed -e 's/[0-9.]//g' )
-      if [ -z "$notInteger" ]
-      then
-        echo "$uid"
-      fi
-    done
-  fi
-}
-
-ExtractAccns() {
-
-  if [ "$needHistory" = false ]
-  then
-    GenerateUidList "$dbase" |
-    while read uid
-    do
-      notInteger=$( echo "$uid" | sed -e 's/[0-9.]//g' )
-      if [ -n "$notInteger" ]
-      then
-        echo "$uid"
-      fi
-    done
-  fi
-}
-
 ExtractPMCIds() {
 
   if [ "$needHistory" = false ]
   then
-    GenerateUidList "$dbase" |
+    GetUIDs |
     while read uid
     do
       case "$uid" in
@@ -1211,7 +1254,7 @@ ExtractPDB() {
 
   if [ "$needHistory" = false ]
   then
-    GenerateUidList "$dbase" |
+    GetUIDs |
     while read uid
     do
       case "$uid" in
@@ -1234,7 +1277,7 @@ ExtractNonPDB() {
 
   if [ "$needHistory" = false ]
   then
-    GenerateUidList "$dbase" |
+    GetUIDs |
     while read uid
     do
       case "$uid" in
@@ -1348,7 +1391,7 @@ LookupSpecialAccessions() {
         ids=$( ExtractPMCIds | fmt -w 1 | sort -n | uniq )
         ;;
       nuc* )
-        nucUidList=$( GenerateUidList "$dbase" )
+        nucUidList=$( GetUIDs )
         anyNonInteger=$( echo "$nucUidList" | sed -e 's/[0-9.]//g' )
         if [ -n "$anyNonInteger" ]
         then
@@ -1373,6 +1416,8 @@ LookupSpecialAccessions() {
           then
             ids=$( echo "$pacres $accres $lcl" | fmt -w 1 | sort -n | uniq )
           fi
+        else
+          ids="$nucUidList"
         fi
         ;;
       protein )
@@ -1386,37 +1431,60 @@ LookupSpecialAccessions() {
         fi
         ;;
       snp )
-        acc=$( ExtractAccns )
-        lcl=$( ExtractUIDs )
-        if [ -n "$acc" ]
+        snpUidList=$( GetUIDs )
+        anyNonInteger=$( echo "$snpUidList" | sed -e 's/[0-9.]//g' )
+        if [ -n "$anyNonInteger" ]
         then
-          query=$( echo "$acc" | PrepareSnpQuery "$fld" )
-          rem=$( esearch -db "$dbase" -query "$query" | efetch -format uid )
-          ids=$( echo "$rem $lcl" | fmt -w 1 | sort -n | uniq )
+          acc=$( echo "$snpUidList" | grep -v '^[0-9]*$' )
+          lcl=$( echo "$snpUidList" | grep '^[0-9]*$' )
+          if [ -n "$acc" ]
+          then
+            query=$( echo "$acc" | PrepareSnpQuery "$fld" )
+            rem=$( esearch -db "$dbase" -query "$query" | efetch -format uid )
+            ids=$( echo "$rem $lcl" | fmt -w 1 | sort -n | uniq )
+          else
+            ids=$( echo "$lcl" | fmt -w 1 | sort -n | uniq )
+          fi
+        else
+          ids=$( echo "$snpUidList" | fmt -w 1 | sort -n | uniq )
         fi
         ;;
       taxonomy )
-        acc=$( ExtractAccns )
-        if [ -n "$acc" ]
+        taxUidList=$( GetUIDs )
+        anyNonInteger=$( echo "$taxUidList" | sed -e 's/[0-9.]//g' )
+        if [ -n "$anyNonInteger" ]
         then
-          echo "${INVT} ERROR: ${LOUD} Taxonomy database does not index sequence accession numbers${INIT}" >&2
+          DisplayError "Taxonomy database does not index sequence accession numbers"
           exit 1
+        else
+          ids=$( echo "$taxUidList" | fmt -w 1 | sort -n | uniq )
         fi
         ;;
     esac
     if [ -n "$fld" ]
     then
-      acc=$( ExtractAccns )
-      lcl=$( ExtractUIDs )
-      if [ -n "$acc" ]
+      otherUidList=$( GetUIDs )
+      anyNonInteger=$( echo "$otherUidList" | sed -e 's/[0-9.]//g' )
+      if [ -n "$anyNonInteger" ]
       then
-        newids=$( echo "$acc" |
-                  PrepareAccnQuery "$fld" "1000" |
-                  RunAccnSearch "10000" )
-        if [ -n "$newids" ]
+        acc=$( echo "$otherUidList" | grep -v '^[0-9]*$' )
+        lcl=$( echo "$otherUidList" | grep '^[0-9]*$' )
+        if [ -n "$acc" ]
         then
-          ids=$( echo "$newids $lcl" | fmt -w 1 | sort -n | uniq )
+          newids=$( echo "$acc" |
+                    PrepareAccnQuery "$fld" "1000" |
+                    RunAccnSearch "10000" )
+          if [ -n "$newids" ]
+          then
+            ids=$( echo "$newids $lcl" | fmt -w 1 | sort -n | uniq )
+          else
+            ids=$( echo "$lcl" | fmt -w 1 | sort -n | uniq )
+          fi
+        else
+          ids=$( echo "$lcl" | fmt -w 1 | sort -n | uniq )
         fi
+      else
+        ids=$( echo "$otherUidList" | fmt -w 1 | sort -n | uniq )
       fi
     fi
   fi
@@ -1489,7 +1557,7 @@ WriteEDirect() {
   if [ -n "$stpx" ]
   then
     # increment step value
-    stpx=$(( $stpx + 1 ))
+    stpx=$(( stpx + 1 ))
     echo "  <Step>${stpx}</Step>"
   fi
   if [ -n "$errx" ]
@@ -1541,3 +1609,29 @@ WriteEDirect() {
 
   echo "</ENTREZ_DIRECT>"
 }
+
+# check for new EDirect version on ftp site
+
+NewerEntrezDirectVersion () {
+
+  newer=$(
+    nquire -lst ftp://ftp.ncbi.nlm.nih.gov/entrez/entrezdirect/versions |
+    cut -d '.' -f 1,2 | sort -t '.' -k 1,1n -k 2,2n | tail -n 1
+  )
+
+  newerWhole=$( echo "$newer" | cut -d '.' -f 1 )
+  newerFract=$( echo "$newer" | cut -d '.' -f 2 )
+
+  currentWhole=$( echo "$version" | cut -d '.' -f 1 )
+  currentFract=$( echo "$version" | cut -d '.' -f 2 )
+
+  if [ "$newerWhole" -gt "$currentWhole" ]
+  then
+    echo "$newer"
+  elif [ "$newerWhole" -eq "$currentWhole" ] && [ "$newerFract" -gt "$currentFract" ]
+  then
+    echo "$newer"
+  else
+    echo ""
+  fi
+}
