@@ -31,9 +31,12 @@
 package eutils
 
 import (
+	"cmp"
 	"encoding/hex"
+	"golang.org/x/text/cases"
+	"golang.org/x/text/language"
 	"html"
-	"sort"
+	"slices"
 	"strconv"
 	"strings"
 	"sync"
@@ -57,6 +60,55 @@ const (
 // removes mixed content tags
 var mfix *strings.Replacer
 
+// lower-case version of combiningAccents
+var lcBadAccents map[string]bool
+
+var invisibleRunes = map[rune]bool{
+	0x00A0: true, // NO-BREAK SPACE
+	0x00AD: true, // SOFT HYPHEN
+	0x034F: true, // COMBINING GRAPHEME JOINER
+	0x061C: true, // ARABIC LETTER MARK
+	0x115F: true, // HANGUL CHOSEONG FILLER
+	0x1160: true, // HANGUL JUNGSEONG FILLER
+	0x17B4: true, // KHMER VOWEL INHERENT AQ
+	0x17B5: true, // KHMER VOWEL INHERENT AA
+	0x180E: true, // MONGOLIAN VOWEL SEPARATOR
+	0x2000: true, // EN QUAD
+	0x2001: true, // EM QUAD
+	0x2002: true, // EN SPACE
+	0x2003: true, // EM SPACE
+	0x2004: true, // THREE-PER-EM SPACE
+	0x2005: true, // FOUR-PER-EM SPACE
+	0x2006: true, // SIX-PER-EM SPACE
+	0x2007: true, // FIGURE SPACE
+	0x2008: true, // PUNCTUATION SPACE
+	0x2009: true, // THIN SPACE
+	0x200A: true, // HAIR SPACE
+	0x200B: true, // ZERO WIDTH SPACE
+	0x200C: true, // ZERO WIDTH NON-JOINER
+	0x200D: true, // ZERO WIDTH JOINER
+	0x200E: true, // LEFT-TO-RIGHT MARK
+	0x200F: true, // RIGHT-TO-LEFT MARK
+	0x202F: true, // NARROW NO-BREAK SPACE
+	0x205F: true, // MEDIUM MATHEMATICAL SPACE
+	0x2060: true, // WORD JOINER
+	0x2061: true, // FUNCTION APPLICATION
+	0x2062: true, // INVISIBLE TIMES
+	0x2063: true, // INVISIBLE SEPARATOR
+	0x2064: true, // INVISIBLE PLUS
+	0x206A: true, // INHIBIT SYMMETRIC SWAPPING
+	0x206B: true, // ACTIVATE SYMMETRIC SWAPPING
+	0x206C: true, // INHIBIT ARABIC FORM SHAPING
+	0x206D: true, // ACTIVATE ARABIC FORM SHAPING
+	0x206E: true, // NATIONAL DIGIT SHAPES
+	0x206F: true, // NOMINAL DIGIT SHAPES
+	0x3000: true, // IDEOGRAPHIC SPACE
+	0x2800: true, // BRAILLE PATTERN BLANK
+	0x3164: true, // HANGUL FILLER
+	0xFEFF: true, // ZERO WIDTH NO-BREAK SPACE
+	0xFFA0: true, // HALFWIDTH HANGUL FILLER
+}
+
 var subScriptRunes = map[rune]rune{
 	0x2080: '0',
 	0x2081: '1',
@@ -1306,129 +1358,32 @@ var ncbi4naToIupac = map[int]string{
 
 // use RelaxString to convert non-alphanumeric characters to spaces
 
-// CleanAuthor is used for citation matching
-func CleanAuthor(str string) string {
+// CleanupAuthor fixes misused letters and accents
+func CleanupAuthor(str string, wrp bool) string {
 
 	if str == "" {
 		return str
 	}
 
-	str = FixMisusedLetters(str, true, true, false)
-	str = TransformAccents(str, false, false)
+	str = strings.Replace(str, "\n", " ", -1)
 
+	str = FixMisusedLetters(str, false, true, false)
+	str = TransformAccents(str, false, false)
 	// convert numeric encoding to apostrophe
 	str = strings.Replace(str, "&#39;", "'", -1)
 	// remove space following apostrophe
 	str = strings.Replace(str, "' ", "'", -1)
-	// then remove apostrophe to match indexing logic
-	str = strings.Replace(str, "'", "", -1)
+	// but leave apostrophe present in result
 
 	if HasUnicodeMarkup(str) {
 		str = RepairUnicodeMarkup(str, SPACE)
 	}
-	if HasAngleBracket(str) {
+	if HasAngleOrAmpersandEncoding(str) {
 		str = RepairTableMarkup(str, SPACE)
-		// if wrp // str = encodeAngleBracketsAndAmpersand(str)
-	}
-	if HasBadSpace(str) {
-		str = CleanupBadSpaces(str)
-	}
-	str = strings.Replace(str, "(", " ", -1)
-	str = strings.Replace(str, ")", " ", -1)
-	if HasAdjacentSpaces(str) {
-		str = CompressRunsOfSpaces(str)
-	}
-	if HasExtraSpaces(str) {
-		str = RemoveExtraSpaces(str)
-	}
-
-	// remove jr and sr suffix
-	if strings.Index(str, " ") != strings.LastIndex(str, " ") {
-		str = strings.TrimSuffix(str, " jr")
-		str = strings.TrimSuffix(str, " Jr")
-		str = strings.TrimSuffix(str, " sr")
-		str = strings.TrimSuffix(str, " Sr")
-	}
-
-	// normalize remaining punctuation
-	str = GenBankToMedlineAuthors(str)
-
-	// truncate to single initial
-	pos := strings.LastIndex(str, " ")
-	if pos > 0 && len(str) > pos+2 {
-		str = str[:pos+2]
-	}
-
-	str = CompressRunsOfSpaces(str)
-	str = strings.TrimSpace(str)
-
-	return str
-}
-
-// CleanJournal is used for citation matching
-func CleanJournal(str string) string {
-
-	if str == "" {
-		return str
-	}
-
-	// convert numeric encoding to apostrophe
-	str = strings.Replace(str, "&#39;", "'", -1)
-	// remove apostrophe, but do not remove trailing space
-	str = strings.Replace(str, "'", "", -1)
-
-	// convert period to space
-	str = strings.Replace(str, ".", " ", -1)
-
-	// convert " & " to " and "
-	str = strings.Replace(str, " &amp; ", " and ", -1)
-
-	words := strings.FieldsFunc(str, func(c rune) bool {
-		// split at non-alphanumeric characters
-		return (!unicode.IsLetter(c) && !unicode.IsDigit(c)) || c > 127
-	})
-	str = strings.Join(words, " ")
-
-	str = CompressRunsOfSpaces(str)
-	str = strings.TrimSpace(str)
-
-	return str
-}
-
-// CleanPage is used for citation matching
-func CleanPage(str string) string {
-
-	if str == "" {
-		return str
-	}
-
-	// keep starting page before hyphen
-	str, _ = SplitInTwoLeft(str, "-")
-
-	str = CompressRunsOfSpaces(str)
-	str = strings.TrimSpace(str)
-
-	return str
-}
-
-// CleanTitle is used for citation matching
-func CleanTitle(str string) string {
-
-	if str == "" {
-		return str
-	}
-
-	str = FixMisusedLetters(str, true, false, true)
-	str = TransformAccents(str, false, false)
-	if HasUnicodeMarkup(str) {
-		str = RepairUnicodeMarkup(str, SPACE)
-	}
-	if HasAngleBracket(str) {
-		str = RepairTableMarkup(str, SPACE)
-		// if wrp // str = encodeAngleBracketsAndAmpersand(str)
-	}
-	if HasAmpOrNotASCII(str) {
-		str = html.UnescapeString(str)
+		str = RemoveHTMLDecorations(str)
+		if wrp {
+			str = encodeAngleBracketsAndAmpersand(str)
+		}
 	}
 	if HasBadSpace(str) {
 		str = CleanupBadSpaces(str)
@@ -1440,34 +1395,6 @@ func CleanTitle(str string) string {
 		str = RemoveExtraSpaces(str)
 	}
 
-	str = strings.Replace(str, "(", " ", -1)
-	str = strings.Replace(str, ")", " ", -1)
-
-	str = strings.Replace(str, "_", " ", -1)
-
-	str = CompressRunsOfSpaces(str)
-	str = strings.TrimSpace(str)
-
-	return str
-}
-
-// RelaxString removes all punctuation from patterns and search text
-func RelaxString(str string) string {
-
-	if str == "" {
-		return str
-	}
-
-	terms := strings.FieldsFunc(str, func(c rune) bool {
-		// split at non-alphanumeric characters
-		return (!unicode.IsLetter(c) && !unicode.IsDigit(c)) || c > 127
-	})
-
-	str = strings.Join(terms, " ")
-
-	str = CompressRunsOfSpaces(str)
-	str = strings.TrimSpace(str)
-
 	return str
 }
 
@@ -1486,14 +1413,26 @@ func CleanupBadSpaces(str string) string {
 			buffer.WriteRune(' ')
 		} else if ch >= 0x0080 && ch <= 0x009F {
 			// absorb control characters
-		} else if ch >= 0x2001 && ch <= 0x200B {
-			// special spaces
+		} else if ch == 0x00A0 {
+			// no-break space
+			buffer.WriteRune(' ')
+		} else if ch >= 0x2002 && ch <= 0x200B {
+			// special spaces (en space through zero width space)
+			buffer.WriteRune(' ')
+		} else if ch == 0x202F {
+			// narrow no-break space
+			buffer.WriteRune(' ')
+		} else if ch == 0x3000 {
+			// ideographic space
 			buffer.WriteRune(' ')
 		} else if ch >= 0xE000 && ch <= 0xF8FF {
-			// unassigned values
+			// unassigned values (private use)
 			buffer.WriteRune('(')
 			buffer.WriteRune('?')
 			buffer.WriteRune(')')
+		} else if ch == 0xFEFF {
+			// zero width no-break space
+			buffer.WriteRune(' ')
 		} else {
 			buffer.WriteRune(ch)
 		}
@@ -1502,6 +1441,38 @@ func CleanupBadSpaces(str string) string {
 	return buffer.String()
 }
 
+// CleanupBasic convert superscripts and subscripts
+func CleanupBasic(str string, wrp bool) string {
+
+	if str == "" {
+		return str
+	}
+
+	str = strings.Replace(str, "\n", " ", -1)
+
+	if HasUnicodeMarkup(str) {
+		str = RepairUnicodeMarkup(str, SPACE)
+	}
+	if HasAngleOrAmpersandEncoding(str) {
+		str = RepairTableMarkup(str, SPACE)
+		str = RemoveHTMLDecorations(str)
+		if wrp {
+			str = encodeAngleBracketsAndAmpersand(str)
+		}
+	}
+	if HasBadSpace(str) {
+		str = CleanupBadSpaces(str)
+	}
+	if HasAdjacentSpaces(str) {
+		str = CompressRunsOfSpaces(str)
+	}
+	if HasExtraSpaces(str) {
+		str = RemoveExtraSpaces(str)
+	}
+
+	return str
+}
+
 // CleanupContents performs optional operations on XML content strings
 func CleanupContents(str string, ascii, amper, mixed bool) string {
 
@@ -1521,12 +1492,12 @@ func CleanupContents(str string, ascii, amper, mixed bool) string {
 		}
 	}
 	if doScript {
-		if mixed && HasAngleBracket(str) {
+		if mixed && HasAngleOrAmpersandEncoding(str) {
 			str = RepairScriptMarkup(str, CONCISE)
 		}
 	}
 	if doMathML {
-		if mixed && HasAngleBracket(str) {
+		if mixed && HasAngleOrAmpersandEncoding(str) {
 			str = RepairMathMLMarkup(str, mathMLFix)
 		}
 	}
@@ -1538,7 +1509,7 @@ func CleanupContents(str string, ascii, amper, mixed bool) string {
 			}
 		}
 		if mixed || amper {
-			if HasAngleBracket(str) {
+			if HasAngleOrAmpersandEncoding(str) {
 				str = RepairScriptMarkup(str, CONCISE)
 				str = RepairTableMarkup(str, SPACE)
 				// call RepairScriptMarkup before RemoveEmbeddedMarkup
@@ -1603,6 +1574,91 @@ func CleanupContents(str string, ascii, amper, mixed bool) string {
 	return str
 }
 
+// CleanupPlain removes embedded mixed-content markup tags
+func CleanupPlain(str string, wrp bool) string {
+
+	if str == "" {
+		return str
+	}
+
+	str = strings.Replace(str, "\n", " ", -1)
+
+	str = RemoveEmbeddedMarkup(str)
+	str = TransformAccents(str, false, false)
+
+	if HasUnicodeMarkup(str) {
+		str = RepairUnicodeMarkup(str, SPACE)
+	}
+	if HasAngleOrAmpersandEncoding(str) {
+		str = RepairTableMarkup(str, SPACE)
+		str = RemoveHTMLDecorations(str)
+		if wrp {
+			str = encodeAngleBracketsAndAmpersand(str)
+		}
+	}
+	if HasBadSpace(str) {
+		str = CleanupBadSpaces(str)
+	}
+	if HasAdjacentSpaces(str) {
+		str = CompressRunsOfSpaces(str)
+	}
+	if HasExtraSpaces(str) {
+		str = RemoveExtraSpaces(str)
+	}
+
+	return str
+}
+
+// CleanupProse converts text to ASCII
+func CleanupProse(str string, wrp bool) string {
+
+	if str == "" {
+		return str
+	}
+
+	str = strings.Replace(str, "\n", " ", -1)
+
+	// CleanupBadSpaces must be called before TransformAccents, to convert Unicode En Space converted from &#x2002;
+	if HasBadSpace(str) {
+		str = CleanupBadSpaces(str)
+	}
+
+	if wrp {
+		str = html.UnescapeString(str)
+	}
+
+	str = RemoveEmbeddedMarkup(str)
+	str = TransformAccents(str, false, false)
+	str = FixMisusedLetters(str, true, false, true)
+	str = TransformAccents(str, false, false)
+
+	if wrp {
+		str = html.EscapeString(str)
+	}
+
+	if HasUnicodeMarkup(str) {
+		str = RepairUnicodeMarkup(str, SPACE)
+	}
+	if HasAngleOrAmpersandEncoding(str) {
+		str = RepairTableMarkup(str, SPACE)
+		str = RemoveHTMLDecorations(str)
+		if wrp {
+			str = encodeAngleBracketsAndAmpersand(str)
+		}
+	}
+	if HasBadSpace(str) {
+		str = CleanupBadSpaces(str)
+	}
+	if HasAdjacentSpaces(str) {
+		str = CompressRunsOfSpaces(str)
+	}
+	if HasExtraSpaces(str) {
+		str = RemoveExtraSpaces(str)
+	}
+
+	return str
+}
+
 // CleanupQuery performs optional operations on XML query strings
 func CleanupQuery(str string, exactMatch, removeBrackets bool) string {
 
@@ -1627,7 +1683,7 @@ func CleanupQuery(str string, exactMatch, removeBrackets bool) string {
 	}
 
 	if removeBrackets {
-		if HasAngleBracket(str) {
+		if HasAngleOrAmpersandEncoding(str) {
 			str = RepairEncodedMarkup(str)
 			str = RepairScriptMarkup(str, SPACE)
 			str = RepairMathMLMarkup(str, SPACE)
@@ -1652,6 +1708,61 @@ func CleanupQuery(str string, exactMatch, removeBrackets bool) string {
 	return str
 }
 
+// CleanupSimple normalizes accented letters, spell Greek letters
+func CleanupSimple(str string, wrp bool) string {
+
+	if str == "" {
+		return str
+	}
+
+	str = strings.Replace(str, "\n", " ", -1)
+
+	str = TransformAccents(str, true, false)
+
+	if HasUnicodeMarkup(str) {
+		str = RepairUnicodeMarkup(str, SPACE)
+	}
+	if HasAngleOrAmpersandEncoding(str) {
+		str = RepairTableMarkup(str, SPACE)
+		str = RemoveHTMLDecorations(str)
+		if wrp {
+			str = encodeAngleBracketsAndAmpersand(str)
+		}
+	}
+	if HasBadSpace(str) {
+		str = CleanupBadSpaces(str)
+	}
+	if HasAdjacentSpaces(str) {
+		str = CompressRunsOfSpaces(str)
+	}
+	if HasExtraSpaces(str) {
+		str = RemoveExtraSpaces(str)
+	}
+
+	return str
+}
+
+// CompareAlphaOrNumericKeys is called by slices.Sort{ed}Func to sort strings in alphabetical or numeric order
+func CompareAlphaOrNumericKeys(i, j string) int {
+
+	if IsAllDigits(i) && IsAllDigits(j) {
+		// numeric sort on strings checks lengths first
+		lni, lnj := len(i), len(j)
+		// shorter string is numerically lower, assuming no leading zeros
+		if lni < lnj {
+			return -1
+		}
+		if lni > lnj {
+			return 1
+		}
+		// same length allows comparison of numeric values without
+		// needing expensive string-to-integer conversion
+	}
+
+	// same length or non-numeric, can now do string comparison on contents
+	return cmp.Compare(i, j)
+}
+
 // CompressRunsOfSpaces turns runs of spaces into a single space
 func CompressRunsOfSpaces(str string) string {
 
@@ -1952,8 +2063,8 @@ func HasAmpOrNotASCII(str string) bool {
 	return false
 }
 
-// HasAngleBracket reports angle brackets or ampersand encodings
-func HasAngleBracket(str string) bool {
+// HasAngleOrAmpersandEncoding reports angle brackets or ampersand encodings
+func HasAngleOrAmpersandEncoding(str string) bool {
 
 	hasAmp := false
 	hasSemi := false
@@ -1969,11 +2080,14 @@ func HasAngleBracket(str string) bool {
 	}
 
 	if hasAmp && hasSemi {
-		if strings.Contains(str, "&lt;") ||
-			strings.Contains(str, "&gt;") ||
-			strings.Contains(str, "&amp;") {
-			return true
-		}
+		/*
+			if strings.Contains(str, "&lt;") ||
+				strings.Contains(str, "&gt;") ||
+				strings.Contains(str, "&amp;") {
+				return true
+			}
+		*/
+		return true
 	}
 
 	return false
@@ -1991,14 +2105,49 @@ func HasBadSpace(str string) bool {
 			if ch >= 0x0080 && ch <= 0x009F {
 				return true
 			}
-			// special spaces
-			if ch >= 0x2001 && ch <= 0x200B {
+			// no-break space
+			if ch == 0x00A0 {
 				return true
 			}
-			// unassigned values
+			// special spaces (en space through zero width space)
+			if ch >= 0x2002 && ch <= 0x200B {
+				return true
+			}
+			// narrow no-break space
+			if ch == 0x202F {
+				return true
+			}
+			// ideographic space
+			if ch == 0x3000 {
+				return true
+			}
+			// unassigned values (private use)
 			if ch >= 0xE000 && ch <= 0xF8FF {
 				return true
 			}
+			// zero width no-break space
+			if ch == 0xFEFF {
+				return true
+			}
+		}
+	}
+
+	return false
+}
+
+// HasCombiningAccent reports on spelled-out combining accents
+func HasCombiningAccent(str string) bool {
+
+	// will search with components of lower-case combiningAccents map
+	str = strings.ToLower(str)
+
+	// do not require whole-phrase context or leading square bracket
+	if strings.Index(str, "combining") >= 0 {
+		for ky := range lcBadAccents {
+			pos := strings.Index(str, ky)
+			if pos >= 0 {
+				return true
+			}
 		}
 	}
 
@@ -2078,6 +2227,42 @@ func HasHyphenOrApostrophe(str string) bool {
 	return false
 }
 
+// HasInvisibleUnicode checks for invisible characters
+func HasInvisibleUnicode(str string) bool {
+
+	for _, ch := range str {
+		if invisibleRunes[ch] {
+			return true
+		}
+	}
+
+	return false
+}
+
+// HasLeadingUnicodeDash checks for Unicode dash at beginning of argument
+func HasLeadingUnicodeDash(str string) (rune, bool) {
+
+	// intentional Unicode dash at beginning of non-command argument must be escaped by a backslash
+
+	for _, ch := range str {
+
+		// test 32-bit rune
+		if ch >= 2010 && ch <= 8213 {
+			return ch, true
+		}
+
+		if ch == '-' {
+			// keep checking for dash after leading hyphen
+			continue
+		}
+
+		// return once past hyphens
+		break
+	}
+
+	return 0, false
+}
+
 // HasSpaceOrHyphen checks for multiple words
 func HasSpaceOrHyphen(str string) bool {
 
@@ -2170,6 +2355,18 @@ func HTMLRepair(str string) (string, bool) {
 	return res, ok
 }
 
+// IsAllCaps matches upper-case
+func IsAllCaps(str string) bool {
+
+	for _, ch := range str {
+		if !unicode.IsUpper(ch) {
+			return false
+		}
+	}
+
+	return true
+}
+
 // IsAllCapsOrDigits matches upper-case or digits
 func IsAllCapsOrDigits(str string) bool {
 
@@ -2206,7 +2403,19 @@ func IsAllDigitsOrPeriod(str string) bool {
 	return true
 }
 
-// IsAllLettersOrDigits matches letters or digits
+// IsAllLetters matches only letters
+func IsAllLetters(str string) bool {
+
+	for _, ch := range str {
+		if !unicode.IsLetter(ch) {
+			return false
+		}
+	}
+
+	return true
+}
+
+// IsAllLettersOrDigits matches letters or digits (added for completeness)
 func IsAllLettersOrDigits(str string) bool {
 
 	for _, ch := range str {
@@ -2218,7 +2427,29 @@ func IsAllLettersOrDigits(str string) bool {
 	return true
 }
 
-// IsNotASCII returns true for any character greater than 7-bits
+// IsMixedCaps matches if word has both upper- and lower-case letters
+// other than initial capital followed by all lower-case
+func IsMixedCaps(str string) bool {
+
+	numLower, nonInitialUpper := 0, 0
+
+	for i, ch := range str {
+		if unicode.IsLower(ch) {
+			numLower++
+		}
+		if i > 0 && unicode.IsUpper(ch) {
+			nonInitialUpper++
+		}
+	}
+
+	if numLower > 0 && nonInitialUpper > 0 {
+		return true
+	}
+
+	return false
+}
+
+// IsNotASCII returns true for any non-ASCII Unicode character greater than 7-bits
 func IsNotASCII(str string) bool {
 
 	for _, ch := range str {
@@ -2338,6 +2569,171 @@ func NeedsTightening(str string) bool {
 	return false
 }
 
+// NormalizeAuthor is used for citation matching
+func NormalizeAuthor(str string) string {
+
+	if str == "" {
+		return str
+	}
+
+	str = FixMisusedLetters(str, true, true, false)
+	str = TransformAccents(str, false, false)
+
+	// convert numeric encoding to apostrophe
+	str = strings.Replace(str, "&#39;", "'", -1)
+	// remove space following apostrophe
+	str = strings.Replace(str, "' ", "'", -1)
+	// then remove apostrophe to match indexing logic
+	str = strings.Replace(str, "'", "", -1)
+
+	if HasUnicodeMarkup(str) {
+		str = RepairUnicodeMarkup(str, SPACE)
+	}
+	if HasAngleOrAmpersandEncoding(str) {
+		str = RepairTableMarkup(str, SPACE)
+		// if wrp // str = encodeAngleBracketsAndAmpersand(str)
+	}
+	if HasBadSpace(str) {
+		str = CleanupBadSpaces(str)
+	}
+	str = strings.Replace(str, "(", " ", -1)
+	str = strings.Replace(str, ")", " ", -1)
+	if HasAdjacentSpaces(str) {
+		str = CompressRunsOfSpaces(str)
+	}
+	if HasExtraSpaces(str) {
+		str = RemoveExtraSpaces(str)
+	}
+
+	// remove jr and sr suffix
+	if strings.Index(str, " ") != strings.LastIndex(str, " ") {
+		str = strings.TrimSuffix(str, " jr")
+		str = strings.TrimSuffix(str, " Jr")
+		str = strings.TrimSuffix(str, " sr")
+		str = strings.TrimSuffix(str, " Sr")
+	}
+
+	// normalize remaining punctuation
+	str = GenBankToMedlineAuthors(str)
+
+	// truncate to single initial
+	pos := strings.LastIndex(str, " ")
+	if pos > 0 && len(str) > pos+2 {
+		str = str[:pos+2]
+	}
+
+	str = CompressRunsOfSpaces(str)
+	str = strings.TrimSpace(str)
+
+	return str
+}
+
+// NormalizeJournal is used for citation matching
+func NormalizeJournal(str string) string {
+
+	if str == "" {
+		return str
+	}
+
+	// convert numeric encoding to apostrophe
+	str = strings.Replace(str, "&#39;", "'", -1)
+	// remove apostrophe, but do not remove trailing space
+	str = strings.Replace(str, "'", "", -1)
+
+	// convert period to space
+	str = strings.Replace(str, ".", " ", -1)
+
+	// convert " & " to " and "
+	str = strings.Replace(str, " &amp; ", " and ", -1)
+
+	words := strings.FieldsFunc(str, func(c rune) bool {
+		// split at non-alphanumeric characters
+		return (!unicode.IsLetter(c) && !unicode.IsDigit(c)) || c > 127
+	})
+
+	// stop words to lower case unless first word of journal name or all capital letters or mixed case
+	var arry []string
+
+	csr := cases.Title(language.English)
+
+	isFirst := true
+	for _, item := range words {
+
+		if !IsAllCaps(item) && !IsMixedCaps(item) {
+			item = strings.ToLower(item)
+			if !IsStopWord(item) || isFirst {
+				item = csr.String(item)
+			}
+		}
+
+		arry = append(arry, item)
+		isFirst = false
+	}
+
+	str = strings.Join(arry, " ")
+
+	str = CompressRunsOfSpaces(str)
+	str = strings.TrimSpace(str)
+
+	return str
+}
+
+// NormalizePage is used for citation matching
+func NormalizePage(str string) string {
+
+	if str == "" {
+		return str
+	}
+
+	// keep starting page before hyphen
+	str, _ = SplitInTwoLeft(str, "-")
+
+	str = CompressRunsOfSpaces(str)
+	str = strings.TrimSpace(str)
+
+	return str
+}
+
+// NormalizeTitle is used for citation matching
+func NormalizeTitle(str string) string {
+
+	if str == "" {
+		return str
+	}
+
+	str = FixMisusedLetters(str, true, false, true)
+	str = TransformAccents(str, false, false)
+	if HasUnicodeMarkup(str) {
+		str = RepairUnicodeMarkup(str, SPACE)
+	}
+	if HasAngleOrAmpersandEncoding(str) {
+		str = RepairTableMarkup(str, SPACE)
+		// if wrp // str = encodeAngleBracketsAndAmpersand(str)
+	}
+	if HasAmpOrNotASCII(str) {
+		str = html.UnescapeString(str)
+	}
+	if HasBadSpace(str) {
+		str = CleanupBadSpaces(str)
+	}
+	if HasAdjacentSpaces(str) {
+		str = CompressRunsOfSpaces(str)
+	}
+	if HasExtraSpaces(str) {
+		str = RemoveExtraSpaces(str)
+	}
+
+	str = strings.Replace(str, "(", " ", -1)
+	str = strings.Replace(str, ")", " ", -1)
+
+	str = strings.Replace(str, "_", " ", -1)
+
+	str = CompressRunsOfSpaces(str)
+	str = strings.TrimSpace(str)
+
+	return str
+}
+
 // ParseIndex populates the XMLFind structure for matching by identifier
 func ParseIndex(indx string) *XMLFind {
 
@@ -2369,7 +2765,7 @@ func PrepareForIndexing(str string, doHomoglyphs, isAuthor, isProse, spellGreek,
 	if HasBadSpace(str) {
 		str = CleanupBadSpaces(str)
 	}
-	if HasAngleBracket(str) {
+	if HasAngleOrAmpersandEncoding(str) {
 		str = RepairEncodedMarkup(str)
 		str = RepairTableMarkup(str, SPACE)
 		str = RepairScriptMarkup(str, SPACE)
@@ -2429,6 +2825,26 @@ func PrepareForIndexing(str string, doHomoglyphs, isAuthor, isProse, spellGreek,
 	return str
 }
 
+// RelaxString removes all punctuation from patterns and search text
+func RelaxString(str string) string {
+
+	if str == "" {
+		return str
+	}
+
+	terms := strings.FieldsFunc(str, func(c rune) bool {
+		// split at non-alphanumeric characters
+		return (!unicode.IsLetter(c) && !unicode.IsDigit(c)) || c > 127
+	})
+
+	str = strings.Join(terms, " ")
+
+	str = CompressRunsOfSpaces(str)
+	str = strings.TrimSpace(str)
+
+	return str
+}
+
 // RemoveEmbeddedMarkup removes internal mixed-content sections
 func RemoveEmbeddedMarkup(str string) string {
 
@@ -2448,6 +2864,67 @@ func RemoveEmbeddedMarkup(str string) string {
 	return buffer.String()
 }
 
+// RemoveCommaOrSemicolon replaces comma or semicolon with space
+func RemoveCommaOrSemicolon(str string) string {
+
+	str = strings.ToLower(str)
+
+	if HasCommaOrSemicolon(str) {
+		str = strings.Replace(str, ",", " ", -1)
+		str = strings.Replace(str, ";", " ", -1)
+		str = CompressRunsOfSpaces(str)
+	}
+	str = strings.TrimSpace(str)
+	str = strings.TrimRight(str, ".?:")
+
+	return str
+}
+
+// RemoveExtraSpaces removes spaces inside parentheses, after hyphen, and before comma
+func RemoveExtraSpaces(str string) string {
+
+	if len(str) < 2 {
+		return str
+	}
+
+	var (
+		buffer strings.Builder
+		prev   rune
+	)
+
+	for _, ch := range str {
+		if prev == '(' && ch == ' ' {
+			ch = '('
+		} else if prev == ' ' && ch == ')' {
+			ch = ')'
+		} else if prev == '-' && ch == ' ' {
+			ch = '-'
+		} else if prev == ' ' && ch == ',' {
+			ch = ','
+		} else if prev != 0 {
+			buffer.WriteRune(prev)
+		}
+		prev = ch
+	}
+
+	buffer.WriteRune(prev)
+
+	return buffer.String()
+}
+
+// RemoveHTMLDecorations removes mixed-content formatting decorations
+func RemoveHTMLDecorations(str string) string {
+
+	// unescape converts &lt;b&gt; to <b>
+	// also &#181; to Greek mu character
+	// also &#x...; where x is followed by a hex value
+	str = html.UnescapeString(str)
+	// remove mixed content tags
+	str = mfix.Replace(str)
+
+	return str
+}
+
 // RepairEncodedMarkup removes ampersand-encoded markup
 func RepairEncodedMarkup(str string) string {
 
@@ -2604,67 +3081,6 @@ func RepairEncodedMarkup(str string) string {
 	return buffer.String()
 }
 
-// RemoveCommaOrSemicolon replaces comma or semicolon with space
-func RemoveCommaOrSemicolon(str string) string {
-
-	str = strings.ToLower(str)
-
-	if HasCommaOrSemicolon(str) {
-		str = strings.Replace(str, ",", " ", -1)
-		str = strings.Replace(str, ";", " ", -1)
-		str = CompressRunsOfSpaces(str)
-	}
-	str = strings.TrimSpace(str)
-	str = strings.TrimRight(str, ".?:")
-
-	return str
-}
-
-// RemoveExtraSpaces removes spaces inside parentheses, after hyphen, and before comma
-func RemoveExtraSpaces(str string) string {
-
-	if len(str) < 2 {
-		return str
-	}
-
-	var (
-		buffer strings.Builder
-		prev   rune
-	)
-
-	for _, ch := range str {
-		if prev == '(' && ch == ' ' {
-			ch = '('
-		} else if prev == ' ' && ch == ')' {
-			ch = ')'
-		} else if prev == '-' && ch == ' ' {
-			ch = '-'
-		} else if prev == ' ' && ch == ',' {
-			ch = ','
-		} else if prev != 0 {
-			buffer.WriteRune(prev)
-		}
-		prev = ch
-	}
-
-	buffer.WriteRune(prev)
-
-	return buffer.String()
-}
-
-// RemoveHTMLDecorations removes mixed-content formatting decorations
-func RemoveHTMLDecorations(str string) string {
-
-	// unescape converts &lt;b&gt; to <b>
-	// also &#181; to Greek mu character
-	// also &#x...; where x is followed by a hex value
-	str = html.UnescapeString(str)
-	// remove mixed content tags
-	str = mfix.Replace(str)
-
-	return str
-}
-
 // RepairMathMLMarkup removes MathML embedded markup symbols
 func RepairMathMLMarkup(str string, policy int) string {
 
@@ -3204,6 +3620,43 @@ func RepairUnicodeMarkup(str string, policy int) string {
 	return buffer.String()
 }
 
+// ReverseString creates a mirror-image of letters in a string
+func ReverseString(str string) string {
+
+	if str == "" {
+		return str
+	}
+
+	runes := []rune(str)
+	for i, j := 0, len(runes)-1; i < j; i, j = i+1, j-1 {
+		runes[i], runes[j] = runes[j], runes[i]
+	}
+	rev := string(runes)
+
+	return rev
+}
+
+// SlidingSlices passes a sliding window over a sequence
+func SlidingSlices(str string, num int) []string {
+
+	if str == "" {
+		return nil
+	}
+
+	var arry []string
+
+	fst, lst, max := 0, num, len(str)+1
+
+	for lst < max {
+		sub := str[fst:lst]
+		arry = append(arry, sub)
+		fst++
+		lst++
+	}
+
+	return arry
+}
+
 // SortStringByWords sorts the individual words in a string
 func SortStringByWords(str string) string {
 
@@ -3212,7 +3665,7 @@ func SortStringByWords(str string) string {
 	// check for multiple words
 	if HasSpaceOrHyphen(str) {
 		flds := strings.Fields(str)
-		sort.Slice(flds, func(i, j int) bool { return flds[i] < flds[j] })
+		slices.Sort(flds)
 		str = strings.Join(flds, " ")
 		str = strings.Replace(str, "-", " ", -1)
 		str = CompressRunsOfSpaces(str)
@@ -3272,6 +3725,34 @@ func TightenParentheses(str string) string {
 	return buffer.String()
 }
 
+// TrimBOM removes leading Byte Order Mark, if present
+func TrimBOM(str string) string {
+
+	if str == "" {
+		return str
+	}
+
+	for i, ch := range str {
+
+		if i == 0 {
+			if ch == 65279 {
+				// next offset is size of BOM
+				continue
+			}
+			// otherwise break out of loop
+			break
+		}
+
+		// trim with size of BOM (should be 3)
+		str = str[i:]
+
+		// break to return trimmed string
+		break
+	}
+
+	return str
+}
+
 // UnicodeToASCII converts nonASCII to ampersand-pound-x encoding
 func UnicodeToASCII(str string) string {
 
@@ -3320,4 +3801,12 @@ func init() {
 		"<sup/>", "",
 		"<sub/>", "",
 	)
+
+	lcBadAccents = make(map[string]bool)
+
+	for ky := range combiningAccents {
+		ky = strings.ToLower(ky)
+		lcBadAccents[ky] = true
+	}
+
 }
